#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Indicator;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Strategy;
#endregion

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    /// <summary>
    /// Trailing sotp within onbarupdate.
    /// </summary>
    [Description("Trailing sotp within onbarupdate.")]
    public class PercentTrailingStop : Strategy
    {
        #region Variables
        // Wizard generated variables
        private double trailStop = 0; // Default setting for TrailStop
        private double prevValue = 0; // Default setting for PrevValue
		private double trailStopPercentage = .02;
        private bool justEntered = false; // Default setting for JustEntered
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the strategy and is called once before any strategy method is called.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose = true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (Close[0] > Open[0] && Position.MarketPosition == MarketPosition.Flat)
			{
				EnterLong();
				justEntered = true;
				Print(Time[0] + " Entered Long " + " symbol " + Instrument.FullName);
			}
			
			if (Position.MarketPosition == MarketPosition.Long && justEntered == true)
			{
				trailStop = Position.AvgPrice - (trailStopPercentage * Position.AvgPrice);
    			prevValue = Position.AvgPrice;
    			justEntered = false;
    			Print(" TRAIL STOP TRACKING: " + trailStop + " symbol " + Instrument.FullName);
			}
			if (Position.MarketPosition == MarketPosition.Long)
			{
				if (High[0] > Position.AvgPrice && High[0] > prevValue)
				{
					trailStop = trailStop + (High[0] - prevValue);
       				prevValue = High[0];
       				Print(" TRAIL STOP RAISED: " + trailStop + " PrevValue " + prevValue + " symbol " + Instrument.FullName);
				}
				Print(Time[0] + " High " + High[0] + " PrevValue " + prevValue + " symbol " + Instrument.FullName);
			}
			if (Low[0] <= trailStop)
			{
				Print(" TRAIL STOP HIT: " + trailStop + " " + Close[0] + " symbol " + Instrument.FullName);
    			// Trailing stop has been hit; do whatever you want here
				ExitLong("","");
   				trailStop = 0;
    			prevValue = 0;
			}
			
				
        }

        #region Properties
        [Description("")]
        [Category("Parameters")]
        public double TrailStop
        {
            get { return trailStop; }
            set { trailStop = Math.Max(0.00, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public double PrevValue
        {
            get { return prevValue; }
            set { prevValue = Math.Max(0.00, value); }
        }

		[Description("")]
        [Category("Parameters")]
        public double TrailStopPercentage
        {
            get { return trailStopPercentage; }
            set { trailStopPercentage = Math.Max(0.00, value); }
        }
		
        [Description("")]
        [Category("Parameters")]
        public bool JustEntered
        {
            get { return justEntered; }
            set { justEntered = value; }
        }
        #endregion
    }
}
